<?php

namespace Drupal\vev_connector\Plugin\media\Source;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\media\Attribute\MediaSource;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaTypeInterface;

/**
 * Script Tag media source.
 */
#[MediaSource(
  id: "script_tag",
  label: new TranslatableMarkup("Script Tag"),
  description: new TranslatableMarkup("Use script tags for reusable media."),
  allowed_field_types: ["text"],
)]
class ScriptTag extends MediaSourceBase {

  /**
   * Key for "Name" metadata attribute.
   *
   * @var string
   */
  const METADATA_ATTRIBUTE_SCRIPT_TAG = 'script_tag';

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    return [
      static::METADATA_ATTRIBUTE_SCRIPT_TAG => $this->t('Script Tag'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type) {
    return parent::createSourceField($type)->set('settings', ['allowed_formats' => ['script_tags']]);
  }

}
