<?php

namespace Drupal\Tests\vev_connector\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\HttpFoundation\Request;

/**
 * Test class with some added functions for testing.
 */
class VevConnectorTest extends KernelTestBase {

  /**
   * Set to TRUE to strict check all configuration saved.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Development\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * The configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The request test content.
   *
   * @var string
   */
  const CONTENT = '{"id":"upload_1234567890","hosting":"123456789","event":"PUBLISH","payload":{"projectId":"123456789-","version":122,"projectTitle":"Webhook API project","dir":"","pages":[{"created":1725980041150,"title":"Vev test page 2","updated":1725980041150,"image":null,"compiled":1726578291130,"compileStatus":3,"viewerUrl":"https://cdn.vev.design/v/12345678","key":"12345678","path":"page-2","project":"123456789-","downloadUrl":"https://storage.googleapis.com/vev-public-temp/webhook-assets%1234567890-1000%2F123456789-%2F100%2F123456781-0%2Fpage2.html"},{"created":1724407269517,"image":null,"title":"Vev test page 1","updated":1725887024873,"compileStatus":3,"viewerUrl":"https://cdn.vev.design/v/12345","compiled":1726581127794,"key":"123456","path":"/","project":"123456789-","downloadUrl":"https://storage.googleapis.com/vev-public-temp/webhook-assets%1234567890-1000%2F123456789-%2F122%2F123456782%2Findex.html"}],"css":[],"js":[],"other":[],"pluginsData":{"vev_essentials":{"title":""},"meta_tags":{"customs":[],"description":""},"social_tags":{"facebookadmins":[],"facebookpages":[],"facebookappid":"132151197500198"},"og_tags":{"optional":{"ogsitename":"Webhook API project","ogdescription":""},"basic":{"ogimage":"https://api.vev.page/image/social/undefined/page/v/102.jpg","ogtype":"website","ogtitle":"","ogurl":"https://dev-drupal-vev.pantheonsite.io/"}}},"workspace":{"art":{"type":0,"value":"🪐"},"workspace":null,"name":"Venus Testing","team":"123456q","key":"q123456"},"team":{"accountKey":"12345","isPublic":true,"updatedBy":"1234567890","icon":{"type":0,"value":"🧩"},"name":"Sandbox ","key":"123123123"}}}';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'media',
    'image',
    'user',
    'field',
    'system',
    'config',
    'file',
    'media_library',
    'filter',
    'token',
    'node',
    'options',
    'text',
    'views',
    'vev_connector',
  ];

  /**
   * The Created media titles.
   *
   * @var string
   */
  const CREATED  = [
    'Webhook API project - Vev test page 1 - 123456789-',
    'Webhook API project - Vev test page 2 - 123456789-',
  ];

  /**
   * The Updated media titles.
   *
   * @var string
   */
  const UPDATED  = [
    'Webhook API project - Vev updated page 1 - 123456789-',
    'Webhook API project - Vev updated page 2 - 123456789-',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installEntitySchema('file');
    $this->installEntitySchema('media');
    $this->installSchema('file', 'file_usage');
    $this->installConfig([
      'field',
      'system',
      'image',
      'file',
      'media',
      'media_library',
      'config',
      'filter',
      'token',
      'node',
      'options',
      'text',
      'views',
      'vev_connector',
    ]);

    $this->configFactory = \Drupal::configFactory();
    $this->configFactory->getEditable('vev_connector.settings')->set('vev_key', '1234567890')->save();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
  }

  /**
   * Test for request.
   */
  public function testVevRequest() {
    $vev_connector_settings = $this->configFactory->get('vev_connector.settings');
    $key = $vev_connector_settings->get('vev_key');
    $http_kernel = $this->container->get('http_kernel');

    // Create request.
    $this->requestFlow($http_kernel, $key, static::CONTENT, static::CREATED);

    // Update request.
    $content = static::CONTENT;
    $content = str_replace('Vev test page', 'Vev updated page', $content);
    $this->requestFlow($http_kernel, $key, $content, static::UPDATED);
  }

  /**
   * Request flow function.
   */
  public function requestFlow($http_kernel, $key, $content, $titles) {
    $request = Request::create('/vev-webhook', 'GET', [], [], [], [], $content);
    $hash = 'sha512=' . hash_hmac('sha512', $request->getContent(), mb_convert_encoding($key, 'UTF-8'));
    $request->headers->set('X-Vev-Signature', $hash);
    $this->assertSame($hash,  $request->headers->get('X-Vev-Signature'));
    $http_kernel->handle($request);

    $medias = $this->entityTypeManager->getStorage('media')->loadByProperties(['bundle' => 'vev_api']);
    $this->assertNotEmpty($medias);

    if ($medias) {
      foreach ($medias as $media) {
        $this->assertTrue(in_array($media->label(), $titles));
      }
    }
  }

}
